#ifndef _TRACEPLAY_COMMON_H
#define _TRACEPLAY_COMMON_H

#include "traceplay.h"

#ifdef __cplusplus
extern "C" {
#endif

#define SURFACE_TYPE_VDP_VID_DEC_ORDER         1
#define SURFACE_TYPE_VDP_VID_DISP_ORDER        2
#define SURFACE_TYPE_VDP_OUT                   3
#define SURFACE_TYPE_GL_TFP_OUT                4
#define SURFACE_TYPE_GL_INTEROP_VID            5
#define SURFACE_TYPE_GL_INTEROP_OUT            6
#define SURFACE_TYPE_GL_WIN                    7
#define SURFACE_TYPE_CUDA_DRIVER_INTEROP_VID   8
#define SURFACE_TYPE_CUDA_DRIVER_INTEROP_OUT   9
#define SURFACE_TYPE_CUDA_TOOLKIT_INTEROP_VID 10
#define SURFACE_TYPE_CUDA_TOOLKIT_INTEROP_OUT 11

extern char * app_name;

extern int      support_opt_scale_size;
extern uint32_t support_opt_gold_surface_type_mask;

extern int      opt_repeat_count;
extern uint32_t opt_gold_every;
extern uint32_t opt_gold_surface_type;

extern Traceplay_Stream_Conf * trace_stream_conf_ptr;
extern Traceplay_Cmds        * trace_cmds_ptr;

extern uint32_t surface_buffer_size;
extern uint8_t *scratch_surface_buffer;
extern uint8_t *surface_buffer;

#define CHECK_PTR(_ptr_) \
  if (!(_ptr_)) { \
      printf("ERROR at %s:%d: %s==NULL\n", __FILE__, __LINE__, #_ptr_); \
      common_exit_error(); \
  }

#define CHECK_ST \
  if (vdp_st != VDP_STATUS_OK) { \
      printf("ERROR at %s:%d: VDPAU error %d\n", __FILE__, __LINE__, vdp_st); \
      common_exit_error(); \
  }

#define CHECK_CU_RESULT \
  if (cr != CUDA_SUCCESS) { \
      printf("Error at %s:%d: CUDA error %d\n", __FILE__, __LINE__, cr); \
      common_exit_error(); \
  }

#define CHECK_CUDA_ERROR \
  if (cr != cudaSuccess) { \
      printf("Error at %s:%d: CUDA error %d\n", __FILE__, __LINE__, cr); \
      common_exit_error(); \
  }

#define COMMON_PARSE_ARG_NOT_PARSED 0
#define COMMON_PARSE_ARG_OK         1
#define COMMON_PARSE_ARG_ERROR      2

#define COMMON_CHECK_ARGS_OK    0
#define COMMON_CHECK_ARGS_ERROR 1

void common_init();
void common_print_help();
int  common_parse_arg(int * argc, char * * * argv);
int  common_check_args();
void common_load_stream();
void common_force_failure_but_dont_exit();
void common_exit_ok();
void common_exit_error();
void common_playback_init(
    VdpDevice           vdp_device,
    VdpGetProcAddress * vdp_get_proc_address,
    int                 need_scratch_surface_buffer
);
void common_do_gold_vdp_video_surface(uint32_t frame_number, VdpVideoSurface surface);
void common_do_gold_vdp_output_surface(uint32_t frame_number, VdpOutputSurface surface);
void common_do_gold_buffer_argb_to_rgb(uint32_t frame_number, uint32_t width, uint32_t height, uint8_t * buffer, size_t buffer_size_bytes, int vinvert);
void common_do_gold_buffer(uint32_t frame_number, char const * format, uint32_t width, uint32_t height, uint8_t const * buffer, size_t buffer_size_bytes);
void common_playback_fini();
void common_nv12x2_to_nv12(uint32_t width, uint32_t height, uint8_t const * src, uint8_t * dest);

#ifdef __cplusplus
}
#endif

#endif

