texture<uchar1, 2, cudaReadModeElementType> luma_top;
texture<uchar1, 2, cudaReadModeElementType> luma_bottom;

extern "C" __global__ void luma_weave(unsigned char *output, unsigned int opitch)
{
    unsigned int pos_x = (blockIdx.x * blockDim.x) + threadIdx.x;
    unsigned int pos_y = (blockIdx.y * blockDim.y) + threadIdx.y;

    unsigned char sample;
    if ((pos_y & 1) == 0) {
        sample = tex2D(luma_top, pos_x, pos_y / 2).x;
    }
    else {
        sample = tex2D(luma_bottom, pos_x, pos_y / 2).x;
    }

    output[(pos_y * opitch) + pos_x] = sample;
}

texture<uchar2, 2, cudaReadModeElementType> chroma_top;
texture<uchar2, 2, cudaReadModeElementType> chroma_bottom;

extern "C" __global__ void chroma_weave(unsigned short *output, unsigned int opitch)
{
    unsigned int pos_x = (blockIdx.x * blockDim.x) + threadIdx.x;
    unsigned int pos_y = (blockIdx.y * blockDim.y) + threadIdx.y;

    uchar2 sample;
    if ((pos_y & 1) == 0) {
        sample = tex2D(chroma_top, pos_x, pos_y / 2);
    }
    else {
        sample = tex2D(chroma_bottom, pos_x, pos_y / 2);
    }

    output[((pos_y * opitch) / 2) + pos_x] = (sample.y << 8) | sample.x;
}

texture<uchar1, 2, cudaReadModeNormalizedFloat> luma;
texture<uchar2, 2, cudaReadModeNormalizedFloat> chroma;

extern "C" __global__ void csc(unsigned int *output, unsigned int opitch, unsigned int out_width, unsigned int out_height)
{
    unsigned int pos_out_x = (blockIdx.x * blockDim.x) + threadIdx.x;
    unsigned int pos_out_y = (blockIdx.y * blockDim.y) + threadIdx.y;

    float pos_in_luma_x = ((float)pos_out_x) / ((float)out_width);
    float pos_in_luma_y = ((float)pos_out_y) / ((float)out_height);

    float pos_in_chroma_x = pos_in_luma_x;
    float pos_in_chroma_y = pos_in_luma_y;

    float y = tex2D(luma, pos_in_luma_x, pos_in_luma_y).x;
    float2 cbcr = tex2D(chroma, pos_in_chroma_x, pos_in_chroma_y);
    float cb = cbcr.x;
    float cr = cbcr.y;

    // 601 CSC matrix
    int r = (int)(0.5f + (255.0f * __saturatef((1.164400f * y) + (0.000000f * cb) + (1.596000f * cr) - 0.874190f)));
    int g = (int)(0.5f + (255.0f * __saturatef((1.164400f * y) - (0.391800f * cb) - (0.813000f * cr) + 0.531702f)));
    int b = (int)(0.5f + (255.0f * __saturatef((1.164400f * y) + (2.017200f * cb) + (0.000000f * cr) - 1.085615f)));
    unsigned int rgba = (r << 16) | (g << 8) | (b << 0);

    output[((pos_out_y * opitch) / 4) + pos_out_x] = rgba;
}

